@@ -8,12 +8,13 @@ from selenium.webdriver.support.ui import WebDriverWait
 from selenium.webdriver.support import expected_conditions as EC
 import unittest, time, logging, os, inspect
 from ..utils.general.Base_Logging import Base_Logging
-# from utils.BrowserStack_Library import BrowserStack_Library
+# from ..utils.general import BrowserStack_Library
 from ..drivers.driverfactory import DriverFactory
 # from utils.Test_Rail import Test_Rail
 # from pages.QCare import PageFactory
 from appium.webdriver.appium_service import AppiumService
 from ..utils.general.Wrapit import Wrapit
+from ..utils.general import Gif_Maker
 
 
 class Borg:
@@ -47,12 +48,15 @@ class Mobile_Base_Page(Borg, unittest.TestCase):
             self.msg_list = []
             self.window_structure = {}
             self.testrail_flag = False
-            self.browserstack_flag = False
+            # self.browserstack_flag = False
             self.test_run_id = None
             self.start_time = None      # pytest have Unresolved attribute reference if not added
             self.exceptions = []
             self.test_env = ""          # pytest have Unresolved attribute reference if not added
-
+            self.rp_logger = None
+            self.gif_file_name = None
+            # self.tesults_flag = False
+            # self.base64_flag = True
             self.reset()
 
         self.driver_obj = DriverFactory()
@@ -76,22 +80,18 @@ class Mobile_Base_Page(Borg, unittest.TestCase):
 
     # def mobile_switch_page(self, page_name, product=None):
     #     """Switch the underlying class to the required Page"""
-    #     from pages.QCare import PageFactory as qcare_PageFactory
-    #     from pages.EMR import PageFactory as emr_PageFactory
-    #     from pages.PatientPortal import PageFactory as pp_PageFactory
+    #     # from pages.QCare import PageFactory as qcare_PageFactory
+    #     # from pages.EMR import PageFactory as emr_PageFactory
+    #     # from pages.PatientPortal import PageFactory as pp_PageFactory
     #     from pages.Website import PageFactory as web_PageFactory
     #     import conf.product_list as product_list
     #
     #     if product is None:
     #         product = self.product
-    #     if product == product_list.QCare:
-    #         self.__class__ = qcare_PageFactory.PageFactory.get_page_object(page_name).__class__
-    #     if product == product_list.EMR:
-    #         self.__class__ = emr_PageFactory.PageFactory.get_page_object(page_name).__class__
     #     if product == product_list.Website:
     #         self.__class__ = web_PageFactory.PageFactory.get_page_object(page_name, platform='mobile').__class__
-    #     if product == product_list.PatientPortal:
-    #         self.__class__ = pp_PageFactory.PageFactory.get_page_object(page_name).__class__
+    #     # if product == product_list.PatientPortal:
+    #     #     self.__class__ = pp_PageFactory.PageFactory.get_page_object(page_name).__class__
 
     def start_appium(self, ip='127.0.0.1', port='4723'):
         """Need to install appium with npm first: npm install -g appium"""
@@ -134,6 +134,10 @@ class Mobile_Base_Page(Borg, unittest.TestCase):
         """Register Tesults with Page"""
         self.tesults_flag = True
 
+    def register_tesults(self):
+        "Register Tesults with Page"
+        self.tesults_flag = True
+
     # def register_browserstack(self):
     #     "Register Browser Stack with Page"
     #     self.browserstack_flag = True
@@ -202,6 +206,10 @@ class Mobile_Base_Page(Borg, unittest.TestCase):
         self.log_name = '[' + self.test_env + ']' + self.testname + '_' + self.timestr + '.log'
         self.log_obj = Base_Logging(log_file_name=self.log_name, level=logging.DEBUG, product=self.product)
 
+    def set_rp_logger(self, rp_pytest_service):
+            "Set the reportportal logger"
+            self.rp_logger = self.log_obj.setup_rp_logging(rp_pytest_service)
+
     def append_latest_image(self, screenshot_name):
         "Get image url list from Browser Stack"
         screenshot_url = self.browserstack_obj.get_latest_screenshot_url()
@@ -210,8 +218,10 @@ class Mobile_Base_Page(Borg, unittest.TestCase):
         image_dict['url'] = screenshot_url
         self.image_url_list.append(image_dict)
 
-    def save_screenshot(self, screenshot_name):
+    def save_screenshot(self, screenshot_name, pre_format="      #Debug screenshot: "):
         "Take a screenshot"
+        # if self.browserstack_flag is True and screenshot_conf.BS_ENABLE_SCREENSHOTS is False:
+        #     return
         if os.path.exists(self.screenshot_dir + os.sep + screenshot_name + '.png'):
             for i in range(1, 100):
                 if os.path.exists(self.screenshot_dir + os.sep + screenshot_name + '_' + str(i) + '.png'):
@@ -220,9 +230,56 @@ class Mobile_Base_Page(Borg, unittest.TestCase):
                     os.rename(self.screenshot_dir + os.sep + screenshot_name + '.png',
                               self.screenshot_dir + os.sep + screenshot_name + '_' + str(i) + '.png')
                     break
-        self.driver.get_screenshot_as_file(self.screenshot_dir + os.sep + screenshot_name + '.png')
-        if self.browserstack_flag is True:
-            self.append_latest_image(screenshot_name)
+        screenshot_name = self.screenshot_dir + os.sep + screenshot_name + '.png'
+        self.driver.get_screenshot_as_file(screenshot_name)
+        # self.conditional_write(flag=True,positive= screenshot_name + '.png',negative='', pre_format=pre_format)
+        if self.rp_logger:
+            self.save_screenshot_reportportal(screenshot_name)
+        # if self.browserstack_flag is True:
+        #     self.append_latest_image(screenshot_name)
+        # if self.tesults_flag is True:
+        #     self.images.append(screenshot_name)
+        # if self.base64_flag is True:
+        #     with open(screenshot_name, "rb") as image_file:
+        #         encoded_string = base64.b64encode(image_file.read()).decode('utf-8')
+        #         self.images.append("data:image/png;base64," + encoded_string)
+        #
+        # # Add screenshot to HTML
+        # # using Jinja2
+        # with open('report.html', 'w') as file:
+        #     template = Template('''
+        #             <html>
+        #             <head>
+        #                 <title>Test Report</title>
+        #             </head>
+        #             <body>
+        #                 {% for image in images %}
+        #                 <img src="{{ image }}" alt="screenshot">
+        #                 {% endfor %}
+        #             </body>
+        #             </html>
+        #             ''')
+        #     file.write(template.render(images=self.images))
+
+
+    def save_screenshot_reportportal(self, image_name):
+        "Method to save image to ReportPortal"
+        try:
+            with open(image_name, "rb") as fh:
+                image = fh.read()
+            screenshot_name = os.path.basename(image_name)
+            self.rp_logger.info(
+                screenshot_name,
+                attachment={
+                    "name": screenshot_name,
+                    "data": image,
+                    "mime": "image/png"
+                },
+            )
+        except Exception as e:
+            self.write("Exception when trying to get rplogger")
+            self.write(str(e))
+            self.exceptions.append("Error when trying to get reportportal logger")
 
     def open(self, url, wait_time=2):
         "Visit the page base_url + url"
@@ -397,6 +454,23 @@ class Mobile_Base_Page(Borg, unittest.TestCase):
             self.appium_service.stop()
             self.write('Stop Appium server')
 
+    def make_gif(self):
+        "Create a gif of all the screenshots within the screenshots directory"
+        # Original code, set a checking case for screenshot option
+        # self.gif_file_name = Gif_Maker.make_gif(self.screenshot_dir, name=self.calling_module)
+        #
+        # return self.gif_file_name
+        if self.screenshot != 'off':
+            self.gif_file_name = Gif_Maker.make_gif(self.screenshot_dir, name=self.calling_module)
+            return self.gif_file_name
+
+    def base_make_gif(self):
+        gif_file = self.make_gif()
+        if self.gif_file_name is not None:
+            self.write("Screenshots & GIF created at %s" % self.screenshot_dir)
+            self.write('************************')
+            return gif_file
+
     def teardown(self):
         "Tears down the driver"
         self.driver.quit()
@@ -425,6 +499,7 @@ class Mobile_Base_Page(Borg, unittest.TestCase):
     #     self.image_url_list = []
     #     self.msg_list = []
 
+
     def wait(self, wait_seconds=5, locator=None):
         "Performs wait for time provided"
         if locator is not None:
@@ -493,6 +568,12 @@ class Mobile_Base_Page(Borg, unittest.TestCase):
             self.write('\n--------FAILURE SUMMARY--------\n')
             for msg in failure_message_list:
                 self.write(msg)
+        if len(self.exceptions) > 0:
+            self.exceptions = list(set(self.exceptions))
+            self.write('\n--------USEFUL EXCEPTION--------\n')
+            for (i, msg) in enumerate(self.exceptions, start=1):
+                self.write(str(i) + "- " + msg)
+        return self.base_make_gif()
 
     def start(self):
         "Dummy method to be over-written by child classes"
